@@ -1,7 +1,7 @@
 #ifndef FINDDIALOG_H
 #define FINDDIALOG_H
 
-#include <QtGui/QDialog>
+#include <QDialog>
 #include <QLineEdit>
 #include <QPushButton>
 
@@ -14,16 +14,19 @@ class FindDialog : public QDialog {
 public:
     FindDialog(QWidget *parent = 0);
     ~FindDialog();
-    QString getFindText();
+//! [findText]
+    QString findText();
+//! [findText]
 
+//! [findClicked]
 public slots:
     void findClicked();
+//! [findClicked]
 
+//! [private members]
 private:
     Ui::FindDialog *m_ui;
-    QPushButton *findButton;
-    QLineEdit *lineEdit;
-    QString findText;
+//! [private members]
 };
 
 #endif // FINDDIALOG_H
