@@ -23,7 +23,7 @@
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** If you are unsure which license is appropriate for your use, please
-** contact the sales department at http://www.qtsoftware.com/contact.
+** contact the sales department at http://qt.nokia.com/contact.
 **
 **************************************************************************/
 
@@ -33,7 +33,6 @@
 #include "pendingchangesdialog.h"
 #include "perforceconstants.h"
 #include "perforceeditor.h"
-#include "perforceoutputwindow.h"
 #include "perforcesubmiteditor.h"
 #include "perforceversioncontrol.h"
 #include "settingspage.h"
@@ -52,6 +51,9 @@
 #include <vcsbase/basevcseditorfactory.h>
 #include <vcsbase/basevcssubmiteditorfactory.h>
 #include <vcsbase/vcsbaseeditor.h>
+#include <vcsbase/vcsbaseoutputwindow.h>
+#include <projectexplorer/project.h>
+#include <projectexplorer/session.h>
 
 #include <QtCore/QtPlugin>
 #include <QtCore/QDebug>
@@ -127,26 +129,28 @@ static const QStringList currentProjectFiles(QString *name)
     return files;
 }
 
-const char * const PerforcePlugin::PERFORCE_MENU = "Perforce.Menu";
-const char * const PerforcePlugin::EDIT = "Perforce.Edit";
-const char * const PerforcePlugin::ADD = "Perforce.Add";
-const char * const PerforcePlugin::DELETE_FILE = "Perforce.Delete";
-const char * const PerforcePlugin::OPENED = "Perforce.Opened";
-const char * const PerforcePlugin::REVERT = "Perforce.Revert";
-const char * const PerforcePlugin::DIFF_CURRENT = "Perforce.DiffCurrent";
-const char * const PerforcePlugin::DIFF_PROJECT = "Perforce.DiffProject";
-const char * const PerforcePlugin::DIFF_ALL = "Perforce.DiffAll";
-const char * const PerforcePlugin::RESOLVE = "Perforce.Resolve";
-const char * const PerforcePlugin::SUBMIT = "Perforce.Submit";
-const char * const PerforcePlugin::PENDING_CHANGES = "Perforce.PendingChanges";
-const char * const PerforcePlugin::DESCRIBE = "Perforce.Describe";
-const char * const PerforcePlugin::ANNOTATE_CURRENT = "Perforce.AnnotateCurrent";
-const char * const PerforcePlugin::ANNOTATE = "Perforce.Annotate";
-const char * const PerforcePlugin::FILELOG_CURRENT = "Perforce.FilelogCurrent";
-const char * const PerforcePlugin::FILELOG = "Perforce.Filelog";
-const char * const PerforcePlugin::SEPARATOR1 = "Perforce.Separator1";
-const char * const PerforcePlugin::SEPARATOR2 = "Perforce.Separator2";
-const char * const PerforcePlugin::SEPARATOR3 = "Perforce.Separator3";
+static const char * const CMD_ID_PERFORCE_MENU = "Perforce.Menu";
+static const char * const CMD_ID_EDIT = "Perforce.Edit";
+static const char * const CMD_ID_ADD = "Perforce.Add";
+static const char * const CMD_ID_DELETE_FILE = "Perforce.Delete";
+static const char * const CMD_ID_OPENED = "Perforce.Opened";
+static const char * const CMD_ID_REVERT = "Perforce.Revert";
+static const char * const CMD_ID_DIFF_CURRENT = "Perforce.DiffCurrent";
+static const char * const CMD_ID_DIFF_PROJECT = "Perforce.DiffProject";
+static const char * const CMD_ID_UPDATE_PROJECT = "Perforce.UpdateProject";
+static const char * const CMD_ID_DIFF_ALL = "Perforce.DiffAll";
+static const char * const CMD_ID_RESOLVE = "Perforce.Resolve";
+static const char * const CMD_ID_SUBMIT = "Perforce.Submit";
+static const char * const CMD_ID_PENDING_CHANGES = "Perforce.PendingChanges";
+static const char * const CMD_ID_DESCRIBE = "Perforce.Describe";
+static const char * const CMD_ID_ANNOTATE_CURRENT = "Perforce.AnnotateCurrent";
+static const char * const CMD_ID_ANNOTATE = "Perforce.Annotate";
+static const char * const CMD_ID_FILELOG_CURRENT = "Perforce.FilelogCurrent";
+static const char * const CMD_ID_FILELOG = "Perforce.Filelog";
+static const char * const CMD_ID_UPDATEALL = "Perforce.UpdateAll";
+static const char * const CMD_ID_SEPARATOR1 = "Perforce.Separator1";
+static const char * const CMD_ID_SEPARATOR2 = "Perforce.Separator2";
+static const char * const CMD_ID_SEPARATOR3 = "Perforce.Separator3";
 
 ////
 // CoreListener
@@ -164,8 +168,6 @@ bool CoreListener::editorAboutToClose(Core::IEditor *editor)
 PerforcePlugin *PerforcePlugin::m_perforcePluginInstance = NULL;
 
 PerforcePlugin::PerforcePlugin() :
-    m_perforceOutputWindow(0),
-    m_settingsPage(0),
     m_editAction(0),
     m_addAction(0),
     m_deleteAction(0),
@@ -173,9 +175,10 @@ PerforcePlugin::PerforcePlugin() :
     m_revertAction(0),
     m_diffCurrentAction(0),
     m_diffProjectAction(0),
+    m_updateProjectAction(0),
     m_diffAllAction(0),
     m_resolveAction(0),
-    m_submitAction(0),    
+    m_submitAction(0),
     m_pendingAction(0),
     m_describeAction(0),
     m_annotateCurrentAction(0),
@@ -183,13 +186,11 @@ PerforcePlugin::PerforcePlugin() :
     m_filelogCurrentAction(0),
     m_filelogAction(0),
     m_submitCurrentLogAction(0),
+    m_updateAllAction(0),
     m_submitActionTriggered(false),
     m_diffSelectedFiles(0),
     m_undoAction(0),
     m_redoAction(0),
-    m_changeTmpFile(0),
-    m_coreListener(0),
-    m_submitEditorFactory(0),
     m_versionControl(0)
 {
 }
@@ -202,8 +203,8 @@ static const VCSBase::VCSBaseSubmitEditorParameters submitParameters = {
 
 bool PerforcePlugin::initialize(const QStringList &arguments, QString *errorMessage)
 {
-    Q_UNUSED(arguments);
-    Q_UNUSED(errorMessage);
+    Q_UNUSED(arguments)
+    Q_UNUSED(errorMessage)
 
     typedef VCSBase::VCSEditorFactory<PerforceEditor> PerforceEditorFactory;
     typedef VCSBase::VCSSubmitEditorFactory<PerforceSubmitEditor> PerforceSubmitEditorFactory;
@@ -216,28 +217,20 @@ bool PerforcePlugin::initialize(const QStringList &arguments, QString *errorMess
     if (QSettings *settings = core->settings())
         m_settings.fromSettings(settings);
 
-    m_perforceOutputWindow = new PerforceOutputWindow(this);
-    addObject(m_perforceOutputWindow);
-
-    m_settingsPage = new SettingsPage;
-    addObject(m_settingsPage);
+    addAutoReleasedObject(new SettingsPage);
 
     // Editor factories
-    m_submitEditorFactory = new PerforceSubmitEditorFactory(&submitParameters);
-    addObject(m_submitEditorFactory);
+    addAutoReleasedObject(new PerforceSubmitEditorFactory(&submitParameters));
 
     static const char *describeSlot = SLOT(describe(QString,QString));
     const int editorCount = sizeof(editorParameters)/sizeof(VCSBase::VCSBaseEditorParameters);
-    for (int i = 0; i < editorCount; i++) {
-        m_editorFactories.push_back(new PerforceEditorFactory(editorParameters + i, this, describeSlot));
-        addObject(m_editorFactories.back());
-    }
+    for (int i = 0; i < editorCount; i++)
+        addAutoReleasedObject(new PerforceEditorFactory(editorParameters + i, this, describeSlot));
 
     m_versionControl = new PerforceVersionControl(this);
-    addObject(m_versionControl);
+    addAutoReleasedObject(m_versionControl);
 
-    m_coreListener = new CoreListener(this);
-    addObject(m_coreListener);
+    addAutoReleasedObject(new CoreListener(this));
 
     //register actions
     Core::ActionManager *am = Core::ICore::instance()->actionManager();
@@ -246,7 +239,7 @@ bool PerforcePlugin::initialize(const QStringList &arguments, QString *errorMess
         am->actionContainer(Core::Constants::M_TOOLS);
 
     Core::ActionContainer *mperforce =
-        am->createMenu(QLatin1String(PERFORCE_MENU));
+        am->createMenu(QLatin1String(CMD_ID_PERFORCE_MENU));
     mperforce->menu()->setTitle(tr("&Perforce"));
     mtools->addMenu(mperforce);
     if (QAction *ma = mperforce->menu()->menuAction()) {
@@ -264,31 +257,31 @@ bool PerforcePlugin::initialize(const QStringList &arguments, QString *errorMess
     Core::Command *command;
     QAction *tmpaction;
 
-    m_editAction = new Core::Utils::ParameterAction(tr("Edit"), tr("Edit \"%1\""), Core::Utils::ParameterAction::EnabledWithParameter, this);
-    command = am->registerAction(m_editAction, PerforcePlugin::EDIT, globalcontext);
+    m_editAction = new Utils::ParameterAction(tr("Edit"), tr("Edit \"%1\""), Utils::ParameterAction::EnabledWithParameter, this);
+    command = am->registerAction(m_editAction, CMD_ID_EDIT, globalcontext);
     command->setAttribute(Core::Command::CA_UpdateText);
     command->setDefaultKeySequence(QKeySequence(tr("Alt+P,Alt+E")));
     command->setDefaultText(tr("Edit File"));
     connect(m_editAction, SIGNAL(triggered()), this, SLOT(openCurrentFile()));
     mperforce->addAction(command);
 
-    m_addAction = new Core::Utils::ParameterAction(tr("Add"), tr("Add \"%1\""), Core::Utils::ParameterAction::EnabledWithParameter, this);
-    command = am->registerAction(m_addAction, PerforcePlugin::ADD, globalcontext);
+    m_addAction = new Utils::ParameterAction(tr("Add"), tr("Add \"%1\""), Utils::ParameterAction::EnabledWithParameter, this);
+    command = am->registerAction(m_addAction, CMD_ID_ADD, globalcontext);
     command->setAttribute(Core::Command::CA_UpdateText);
     command->setDefaultKeySequence(QKeySequence(tr("Alt+P,Alt+A")));
     command->setDefaultText(tr("Add File"));
     connect(m_addAction, SIGNAL(triggered()), this, SLOT(addCurrentFile()));
     mperforce->addAction(command);
 
-    m_deleteAction = new Core::Utils::ParameterAction(tr("Delete"), tr("Delete \"%1\""), Core::Utils::ParameterAction::EnabledWithParameter, this);
-    command = am->registerAction(m_deleteAction, PerforcePlugin::DELETE_FILE, globalcontext);
+    m_deleteAction = new Utils::ParameterAction(tr("Delete"), tr("Delete \"%1\""), Utils::ParameterAction::EnabledWithParameter, this);
+    command = am->registerAction(m_deleteAction, CMD_ID_DELETE_FILE, globalcontext);
     command->setAttribute(Core::Command::CA_UpdateText);
     command->setDefaultText(tr("Delete File"));
     connect(m_deleteAction, SIGNAL(triggered()), this, SLOT(deleteCurrentFile()));
     mperforce->addAction(command);
 
-    m_revertAction = new Core::Utils::ParameterAction(tr("Revert"), tr("Revert \"%1\""), Core::Utils::ParameterAction::EnabledWithParameter, this);
-    command = am->registerAction(m_revertAction, PerforcePlugin::REVERT, globalcontext);
+    m_revertAction = new Utils::ParameterAction(tr("Revert"), tr("Revert \"%1\""), Utils::ParameterAction::EnabledWithParameter, this);
+    command = am->registerAction(m_revertAction, CMD_ID_REVERT, globalcontext);
     command->setAttribute(Core::Command::CA_UpdateText);
     command->setDefaultKeySequence(QKeySequence(tr("Alt+P,Alt+R")));
     command->setDefaultText(tr("Revert File"));
@@ -300,23 +293,24 @@ bool PerforcePlugin::initialize(const QStringList &arguments, QString *errorMess
     command = am->registerAction(tmpaction, QLatin1String("Perforce.Sep.Edit"), globalcontext);
     mperforce->addAction(command);
 
-    m_diffCurrentAction = new Core::Utils::ParameterAction(tr("Diff Current File"), tr("Diff \"%1\""), Core::Utils::ParameterAction::EnabledWithParameter, this);
-    command = am->registerAction(m_diffCurrentAction, PerforcePlugin::DIFF_CURRENT, globalcontext);
+    m_diffCurrentAction = new Utils::ParameterAction(tr("Diff Current File"), tr("Diff \"%1\""), Utils::ParameterAction::EnabledWithParameter, this);
+    command = am->registerAction(m_diffCurrentAction, CMD_ID_DIFF_CURRENT, globalcontext);
     command->setAttribute(Core::Command::CA_UpdateText);
     command->setDefaultText(tr("Diff Current File"));
     connect(m_diffCurrentAction, SIGNAL(triggered()), this, SLOT(diffCurrentFile()));
     mperforce->addAction(command);
 
-    m_diffProjectAction = new Core::Utils::ParameterAction(tr("Diff Current Project/Session"), tr("Diff Project \"%1\""), Core::Utils::ParameterAction::EnabledWithParameter, this);
-    command = am->registerAction(m_diffProjectAction, PerforcePlugin::DIFF_PROJECT, globalcontext);
+    const QString diffProjectDefaultText = tr("Diff Current Project/Session");
+    m_diffProjectAction = new Utils::ParameterAction(diffProjectDefaultText, tr("Diff Project \"%1\""), Utils::ParameterAction::AlwaysEnabled, this);
+    command = am->registerAction(m_diffProjectAction, CMD_ID_DIFF_PROJECT, globalcontext);
     command->setAttribute(Core::Command::CA_UpdateText);
     command->setDefaultKeySequence(QKeySequence(tr("Alt+P,Alt+D")));
-    command->setDefaultText(tr("Diff Current Project/Session"));
+    command->setDefaultText(diffProjectDefaultText);
     connect(m_diffProjectAction, SIGNAL(triggered()), this, SLOT(diffCurrentProject()));
     mperforce->addAction(command);
 
     m_diffAllAction = new QAction(tr("Diff Opened Files"), this);
-    command = am->registerAction(m_diffAllAction, PerforcePlugin::DIFF_ALL, globalcontext);
+    command = am->registerAction(m_diffAllAction, CMD_ID_DIFF_ALL, globalcontext);
     connect(m_diffAllAction, SIGNAL(triggered()), this, SLOT(diffAllOpened()));
     mperforce->addAction(command);
 
@@ -326,46 +320,54 @@ bool PerforcePlugin::initialize(const QStringList &arguments, QString *errorMess
     mperforce->addAction(command);
 
     m_openedAction = new QAction(tr("Opened"), this);
-    command = am->registerAction(m_openedAction, PerforcePlugin::OPENED, globalcontext);
+    command = am->registerAction(m_openedAction, CMD_ID_OPENED, globalcontext);
     command->setDefaultKeySequence(QKeySequence(tr("Alt+P,Alt+O")));
     connect(m_openedAction, SIGNAL(triggered()), this, SLOT(printOpenedFileList()));
     mperforce->addAction(command);
 
     m_submitAction = new QAction(tr("Submit Project"), this);
-    command = am->registerAction(m_submitAction, PerforcePlugin::SUBMIT, globalcontext);
+    command = am->registerAction(m_submitAction, CMD_ID_SUBMIT, globalcontext);
     command->setDefaultKeySequence(QKeySequence(tr("Alt+P,Alt+S")));
     connect(m_submitAction, SIGNAL(triggered()), this, SLOT(submit()));
     mperforce->addAction(command);
 
     m_pendingAction = new QAction(tr("Pending Changes..."), this);
-    command = am->registerAction(m_pendingAction, PerforcePlugin::PENDING_CHANGES, globalcontext);
+    command = am->registerAction(m_pendingAction, CMD_ID_PENDING_CHANGES, globalcontext);
     connect(m_pendingAction, SIGNAL(triggered()), this, SLOT(printPendingChanges()));
     mperforce->addAction(command);
 
+    const QString updateProjectDefaultText = tr("Update Current Project/Session");
+    m_updateProjectAction = new Utils::ParameterAction(updateProjectDefaultText, tr("Update Project \"%1\""), Utils::ParameterAction::AlwaysEnabled, this);
+    command = am->registerAction(m_updateProjectAction, CMD_ID_UPDATE_PROJECT, globalcontext);
+    command->setDefaultText(updateProjectDefaultText);
+    command->setAttribute(Core::Command::CA_UpdateText);
+    connect(m_updateProjectAction, SIGNAL(triggered()), this, SLOT(updateCurrentProject()));
+    mperforce->addAction(command);
+
     tmpaction = new QAction(this);
     tmpaction->setSeparator(true);
     command = am->registerAction(tmpaction, QLatin1String("Perforce.Sep.Changes"), globalcontext);
     mperforce->addAction(command);
 
     m_describeAction = new QAction(tr("Describe..."), this);
-    command = am->registerAction(m_describeAction, PerforcePlugin::DESCRIBE, globalcontext);
+    command = am->registerAction(m_describeAction, CMD_ID_DESCRIBE, globalcontext);
     connect(m_describeAction, SIGNAL(triggered()), this, SLOT(describeChange()));
     mperforce->addAction(command);
 
-    m_annotateCurrentAction = new Core::Utils::ParameterAction(tr("Annotate Current File"), tr("Annotate \"%1\""), Core::Utils::ParameterAction::EnabledWithParameter, this);
-    command = am->registerAction(m_annotateCurrentAction, PerforcePlugin::ANNOTATE_CURRENT, globalcontext);
+    m_annotateCurrentAction = new Utils::ParameterAction(tr("Annotate Current File"), tr("Annotate \"%1\""), Utils::ParameterAction::EnabledWithParameter, this);
+    command = am->registerAction(m_annotateCurrentAction, CMD_ID_ANNOTATE_CURRENT, globalcontext);
     command->setAttribute(Core::Command::CA_UpdateText);
     command->setDefaultText(tr("Annotate Current File"));
     connect(m_annotateCurrentAction, SIGNAL(triggered()), this, SLOT(annotateCurrentFile()));
     mperforce->addAction(command);
 
     m_annotateAction = new QAction(tr("Annotate..."), this);
-    command = am->registerAction(m_annotateAction, PerforcePlugin::ANNOTATE, globalcontext);
+    command = am->registerAction(m_annotateAction, CMD_ID_ANNOTATE, globalcontext);
     connect(m_annotateAction, SIGNAL(triggered()), this, SLOT(annotate()));
     mperforce->addAction(command);
 
-    m_filelogCurrentAction = new Core::Utils::ParameterAction(tr("Filelog Current File"), tr("Filelog \"%1\""), Core::Utils::ParameterAction::EnabledWithParameter, this);
-    command = am->registerAction(m_filelogCurrentAction, PerforcePlugin::FILELOG_CURRENT, globalcontext);
+    m_filelogCurrentAction = new Utils::ParameterAction(tr("Filelog Current File"), tr("Filelog \"%1\""), Utils::ParameterAction::EnabledWithParameter, this);
+    command = am->registerAction(m_filelogCurrentAction, CMD_ID_FILELOG_CURRENT, globalcontext);
     command->setAttribute(Core::Command::CA_UpdateText);
     command->setDefaultKeySequence(QKeySequence(tr("Alt+P,Alt+F")));
     command->setDefaultText(tr("Filelog Current File"));
@@ -373,10 +375,15 @@ bool PerforcePlugin::initialize(const QStringList &arguments, QString *errorMess
     mperforce->addAction(command);
 
     m_filelogAction = new QAction(tr("Filelog..."), this);
-    command = am->registerAction(m_filelogAction, PerforcePlugin::FILELOG, globalcontext);
+    command = am->registerAction(m_filelogAction, CMD_ID_FILELOG, globalcontext);
     connect(m_filelogAction, SIGNAL(triggered()), this, SLOT(filelog()));
     mperforce->addAction(command);
 
+    m_updateAllAction = new QAction(tr("Update All"), this);
+    command = am->registerAction(m_updateAllAction, CMD_ID_UPDATEALL, globalcontext);
+    connect(m_updateAllAction, SIGNAL(triggered()), this, SLOT(updateAll()));
+    mperforce->addAction(command);
+
     m_submitCurrentLogAction = new QAction(VCSBase::VCSBaseSubmitEditor::submitIcon(), tr("Submit"), this);
     command = am->registerAction(m_submitCurrentLogAction, Constants::SUBMIT_CURRENT, perforcesubmitcontext);
     connect(m_submitCurrentLogAction, SIGNAL(triggered()), this, SLOT(submitCurrentLog()));
@@ -444,18 +451,11 @@ void PerforcePlugin::revertCurrentFile()
             return;
     }
 
-    Core::FileManager *fm = Core::ICore::instance()->fileManager();
-    QList<Core::IFile *> files = fm->managedFiles(fileName);
-    foreach (Core::IFile *file, files) {
-        fm->blockFileChange(file);
-    }
+    Core::FileChangeBlocker fcb(fileName);
+    fcb.setModifiedReload(true);
     PerforceResponse result2 = runP4Cmd(QStringList() << QLatin1String("revert") << fileName, QStringList(), CommandToWindow|StdOutToWindow|StdErrToWindow|ErrorToWindow);
-    Core::IFile::ReloadBehavior tempBehavior =
-            Core::IFile::ReloadAll;
-    foreach (Core::IFile *file, files) {
-        file->modified(&tempBehavior);
-        fm->unblockFileChange(file);
-    }
+    if (!result2.error)
+        m_versionControl->emitFilesChanged(QStringList(fileName));
 }
 
 void PerforcePlugin::diffCurrentFile()
@@ -475,6 +475,53 @@ void PerforcePlugin::diffAllOpened()
     p4Diff(QStringList());
 }
 
+// Add the directory of a project to a list of p4 directory specifications
+// ("path/...")
+static inline void addProjectP4Directory(const ProjectExplorer::Project *p, QStringList *p4dirs)
+{
+    if (const Core::IFile *file =  p->file()) {
+        const QFileInfo fi(file->fileName());
+        QString p4dir = fi.absolutePath();
+        if (!p4dir.isEmpty()) {
+            p4dir += QDir::separator();
+            p4dir += QLatin1String("...");
+            p4dirs->push_back(p4dir);
+        }
+    }
+}
+
+void PerforcePlugin::updateCurrentProject()
+{
+    if (!m_projectExplorer)
+        return;
+    // Compile a list of project directories
+    QStringList p4Directories;
+    if (const ProjectExplorer::Project *proj = m_projectExplorer->currentProject()) {
+        addProjectP4Directory(proj, &p4Directories);
+    } else {
+        if (const ProjectExplorer::SessionManager *session = m_projectExplorer->session())
+            foreach(const ProjectExplorer::Project *proj, session->projects())
+                addProjectP4Directory(proj, &p4Directories);
+    }
+    if (!p4Directories.empty())
+        updateCheckout(p4Directories);
+}
+
+void PerforcePlugin::updateAll()
+{
+    updateCheckout();
+}
+
+void PerforcePlugin::updateCheckout(const QStringList &dirs)
+{
+    QStringList args(QLatin1String("sync"));
+    args.append(dirs);
+    const PerforceResponse resp = runP4Cmd(args, QStringList(), CommandToWindow|StdOutToWindow|StdErrToWindow|ErrorToWindow);
+    if (!dirs.empty())
+        foreach(const QString &dir, dirs)
+            m_versionControl->emitRepositoryChanged(dir);
+}
+
 void PerforcePlugin::printOpenedFileList()
 {
     Core::IEditor *e = Core::EditorManager::instance()->currentEditor();
@@ -490,32 +537,33 @@ void PerforcePlugin::submit()
 
     QString errorMessage;
     if (!checkP4Configuration(&errorMessage)) {
-        showOutput(errorMessage, true);
+        VCSBase::VCSBaseOutputWindow::instance()->appendError(errorMessage);
         return;
     }
 
-    if (m_changeTmpFile) {
-        showOutput(tr("Another submit is currently executed."), true);
-        m_perforceOutputWindow->popup(false);
+    if (isCommitEditorOpen()) {
+        VCSBase::VCSBaseOutputWindow::instance()->appendWarning(tr("Another submit is currently executed."));
         return;
     }
 
-    m_changeTmpFile = new QTemporaryFile(this);
-    if (!m_changeTmpFile->open()) {
-        showOutput(tr("Cannot create temporary file."), true);
-        cleanChangeTmpFile();
+    QTemporaryFile changeTmpFile;
+    changeTmpFile.setAutoRemove(false);
+    if (!changeTmpFile.open()) {
+        VCSBase::VCSBaseOutputWindow::instance()->appendError(tr("Cannot create temporary file."));
+        cleanCommitMessageFile();
         return;
     }
 
     PerforceResponse result = runP4Cmd(QStringList()<< QLatin1String("change") << QLatin1String("-o"), QStringList(),
                                        CommandToWindow|StdErrToWindow|ErrorToWindow);
     if (result.error) {
-        cleanChangeTmpFile();
+        cleanCommitMessageFile();
         return;
     }
 
-    m_changeTmpFile->write(result.stdOut.toAscii());
-    m_changeTmpFile->seek(0);
+    m_commitMessageFileName = changeTmpFile.fileName();
+    changeTmpFile.write(result.stdOut.toAscii());
+    changeTmpFile.close();
 
     // Assemble file list of project
     QString name;
@@ -523,7 +571,7 @@ void PerforcePlugin::submit()
     PerforceResponse result2 = runP4Cmd(QStringList(QLatin1String("fstat")), nativeFiles,
                                         CommandToWindow|StdErrToWindow|ErrorToWindow);
     if (result2.error) {
-        cleanChangeTmpFile();
+        cleanCommitMessageFile();
         return;
     }
 
@@ -534,12 +582,12 @@ void PerforcePlugin::submit()
             depotFileNames.append(line.mid(14));
     }
     if (depotFileNames.isEmpty()) {
-        showOutput(tr("Project has no files"));
-        cleanChangeTmpFile();
+        VCSBase::VCSBaseOutputWindow::instance()->appendWarning(tr("Project has no files"));
+        cleanCommitMessageFile();
         return;
     }
 
-    openPerforceSubmitEditor(m_changeTmpFile->fileName(), depotFileNames);
+    openPerforceSubmitEditor(m_commitMessageFileName, depotFileNames);
 }
 
 Core::IEditor *PerforcePlugin::openPerforceSubmitEditor(const QString &fileName, const QStringList &depotFileNames)
@@ -547,8 +595,7 @@ Core::IEditor *PerforcePlugin::openPerforceSubmitEditor(const QString &fileName,
     Core::EditorManager *editorManager = Core::EditorManager::instance();
     Core::IEditor *editor = editorManager->openEditor(fileName, Constants::PERFORCESUBMITEDITOR_KIND);
     editorManager->ensureEditorManagerVisible();
-    PerforceSubmitEditor *submitEditor = dynamic_cast<PerforceSubmitEditor*>(editor);
-    QTC_ASSERT(submitEditor, return 0);
+    PerforceSubmitEditor *submitEditor = static_cast<PerforceSubmitEditor*>(editor);
     submitEditor->restrictToProjectFiles(depotFileNames);
     submitEditor->registerActions(m_undoAction, m_redoAction, m_submitCurrentLogAction, m_diffSelectedFiles);
     connect(submitEditor, SIGNAL(diffSelectedFiles(QStringList)), this, SLOT(slotDiff(QStringList)));
@@ -597,9 +644,11 @@ void PerforcePlugin::annotate(const QString &fileName)
     const PerforceResponse result = runP4Cmd(args, QStringList(),
                                              CommandToWindow|StdErrToWindow|ErrorToWindow, codec);
     if (!result.error) {
+        const int lineNumber = VCSBase::VCSBaseEditor::lineNumberOfCurrentEditor(fileName);
         const QFileInfo fi(fileName);
-        showOutputInEditor(tr("p4 annotate %1").arg(fi.fileName()),
-            result.stdOut, VCSBase::AnnotateOutput, codec);
+        Core::IEditor *ed = showOutputInEditor(tr("p4 annotate %1").arg(fi.fileName()),
+                                               result.stdOut, VCSBase::AnnotateOutput, codec);
+        VCSBase::VCSBaseEditor::gotoLineOfEditor(ed, lineNumber);
     }
 }
 
@@ -643,20 +692,31 @@ void PerforcePlugin::updateActions()
     m_diffCurrentAction->setParameter(baseName);
     m_annotateCurrentAction->setParameter(baseName);
     m_filelogCurrentAction->setParameter(baseName);
-    
+
+    // The project actions refer to the current project or the projects
+    // of a session, provided there are any.
     if (m_projectExplorer && m_projectExplorer->currentProject()) {
-        m_diffProjectAction->setParameter(m_projectExplorer->currentProject()->name());
-        m_submitAction->setEnabled(true);
+        const QString currentProjectName = m_projectExplorer->currentProject()->name();
+        m_updateProjectAction->setParameter(currentProjectName);
+        m_diffProjectAction->setParameter(currentProjectName);
+        m_updateProjectAction->setEnabled(true);
+        m_diffProjectAction->setEnabled(true);
     } else {
+        // Nope, either all projects of a session or none
+        m_updateProjectAction->setParameter(QString());
         m_diffProjectAction->setParameter(QString());
-        m_submitAction->setEnabled(false);
+        const bool enabled = m_projectExplorer && m_projectExplorer->session() && !m_projectExplorer->session()->projects().empty();
+        m_updateProjectAction->setEnabled(enabled);
+        m_diffProjectAction->setEnabled(enabled);
     }
+
     m_diffAllAction->setEnabled(true);
     m_openedAction->setEnabled(true);
     m_describeAction->setEnabled(true);
     m_annotateAction->setEnabled(true);
     m_filelogAction->setEnabled(true);
     m_pendingAction->setEnabled(true);
+    m_updateAllAction->setEnabled(true);
 }
 
 bool PerforcePlugin::managesDirectory(const QString &directory) const
@@ -719,6 +779,15 @@ bool PerforcePlugin::vcsDelete(const QString &fileName)
     return !(result.error && result2.error);
 }
 
+static QString formatCommand(const QString &cmd, const QStringList &args)
+{
+    const QChar blank = QLatin1Char(' ');
+    QString command = cmd;
+    command += blank;
+    command += args.join(QString(blank));
+    return PerforcePlugin::tr("Executing: %1\n").arg(command);
+}
+
 PerforceResponse PerforcePlugin::runP4Cmd(const QStringList &args,
                                           const QStringList &extraArgs,
                                           unsigned logFlags,
@@ -728,8 +797,9 @@ PerforceResponse PerforcePlugin::runP4Cmd(const QStringList &args,
         qDebug() << "PerforcePlugin::runP4Cmd" << args << extraArgs << debugCodec(outputCodec);
     PerforceResponse response;
     response.error = true;
+    VCSBase::VCSBaseOutputWindow *outputWindow = VCSBase::VCSBaseOutputWindow::instance();
     if (!checkP4Configuration(&response.message)) {
-        m_perforceOutputWindow->append(response.message, true);
+        outputWindow->appendError(response.message);
         return response;
     }
 
@@ -752,17 +822,11 @@ PerforceResponse PerforcePlugin::runP4Cmd(const QStringList &args,
     }
     actualArgs << args;
 
-    if (logFlags & CommandToWindow) {
-        QString command = m_settings.p4Command();
-        command += blank;
-        command += actualArgs.join(QString(blank));
-        const QString timeStamp = QTime::currentTime().toString(QLatin1String("HH:mm"));
-        const QString outputText = tr("%1 Executing: %2\n").arg(timeStamp, command);
-        showOutput(outputText, false);
-    }
+    if (logFlags & CommandToWindow)
+        outputWindow->appendCommand(formatCommand(m_settings.p4Command(), actualArgs));
 
     // Run, connect stderr to the output window
-    Core::Utils::SynchronousProcess process;
+    Utils::SynchronousProcess process;
     process.setTimeout(p4Timeout);
     process.setStdOutCodec(outputCodec);
     process.setEnvironment(environment());
@@ -770,16 +834,16 @@ PerforceResponse PerforcePlugin::runP4Cmd(const QStringList &args,
     // connect stderr to the output window if desired
     if (logFlags & StdErrToWindow) {
         process.setStdErrBufferedSignalsEnabled(true);
-        connect(&process, SIGNAL(stdErrBuffered(QString,bool)), m_perforceOutputWindow, SLOT(append(QString,bool)));
+        connect(&process, SIGNAL(stdErrBuffered(QString,bool)), outputWindow, SLOT(append(QString)));
     }
 
     // connect stdout to the output window if desired
     if (logFlags & StdOutToWindow) {
         process.setStdOutBufferedSignalsEnabled(true);
-        connect(&process, SIGNAL(stdOutBuffered(QString,bool)), m_perforceOutputWindow, SLOT(append(QString,bool)));
+        connect(&process, SIGNAL(stdOutBuffered(QString,bool)), outputWindow, SLOT(append(QString)));
     }
 
-    const Core::Utils::SynchronousProcessResponse sp_resp = process.run(m_settings.p4Command(), actualArgs);
+    const Utils::SynchronousProcessResponse sp_resp = process.run(m_settings.p4Command(), actualArgs);
     if (Perforce::Constants::debug)
         qDebug() << sp_resp;
 
@@ -787,19 +851,19 @@ PerforceResponse PerforcePlugin::runP4Cmd(const QStringList &args,
     response.stdErr = sp_resp.stdErr;
     response.stdOut = sp_resp.stdOut;
     switch (sp_resp.result) {
-    case Core::Utils::SynchronousProcessResponse::Finished:
+    case Utils::SynchronousProcessResponse::Finished:
         response.error = false;
         break;
-    case Core::Utils::SynchronousProcessResponse::FinishedError:
+    case Utils::SynchronousProcessResponse::FinishedError:
         response.message = tr("The process terminated with exit code %1.").arg(sp_resp.exitCode);
         break;
-    case Core::Utils::SynchronousProcessResponse::TerminatedAbnormally:
+    case Utils::SynchronousProcessResponse::TerminatedAbnormally:
         response.message = tr("The process terminated abnormally.");
         break;
-    case Core::Utils::SynchronousProcessResponse::StartFailed:
+    case Utils::SynchronousProcessResponse::StartFailed:
         response.message = tr("Could not start perforce '%1'. Please check your settings in the preferences.").arg(m_settings.p4Command());
         break;
-    case Core::Utils::SynchronousProcessResponse::Hang:
+    case Utils::SynchronousProcessResponse::Hang:
         response.message = tr("Perforce did not respond within timeout limit (%1 ms).").arg(p4Timeout );
         break;
     }
@@ -807,7 +871,7 @@ PerforceResponse PerforcePlugin::runP4Cmd(const QStringList &args,
         if (Perforce::Constants::debug)
             qDebug() << response.message;
         if (logFlags & ErrorToWindow)
-            m_perforceOutputWindow->append(response.message, true);
+            outputWindow->appendError(response.message);
     }
     return response;
 }
@@ -821,8 +885,7 @@ Core::IEditor * PerforcePlugin::showOutputInEditor(const QString& title, const Q
     if (Perforce::Constants::debug)
         qDebug() << "PerforcePlugin::showOutputInEditor" << title << kind <<  "Size= " << output.size() <<  " Type=" << editorType << debugCodec(codec);
     QString s = title;
-    Core::IEditor *editor = Core::EditorManager::instance()->
-        newFile(kind, &s, output.toLocal8Bit());
+    Core::IEditor *editor = Core::EditorManager::instance()->openEditorWithContents(kind, &s, output);
     PerforceEditor *e = qobject_cast<PerforceEditor*>(editor->widget());
     if (!e)
         return 0;
@@ -917,19 +980,22 @@ void PerforcePlugin::submitCurrentLog()
     em->closeEditors(QList<Core::IEditor*>() << em->currentEditor());
 }
 
-void PerforcePlugin::cleanChangeTmpFile()
+void PerforcePlugin::cleanCommitMessageFile()
 {
-    if (m_changeTmpFile) {
-        if (m_changeTmpFile->isOpen())
-            m_changeTmpFile->close();
-        delete m_changeTmpFile;
-        m_changeTmpFile = 0;
+    if (!m_commitMessageFileName.isEmpty()) {
+        QFile::remove(m_commitMessageFileName);
+        m_commitMessageFileName.clear();
     }
 }
 
+bool PerforcePlugin::isCommitEditorOpen() const
+{
+    return !m_commitMessageFileName.isEmpty();
+}
+
 bool PerforcePlugin::editorAboutToClose(Core::IEditor *editor)
 {
-    if (!m_changeTmpFile || !editor)
+    if (!editor || !isCommitEditorOpen())
         return true;
     Core::ICore *core = Core::ICore::instance();
     Core::IFile *fileIFace = editor->file();
@@ -939,40 +1005,52 @@ bool PerforcePlugin::editorAboutToClose(Core::IEditor *editor)
     if (!perforceEditor)
         return true;
     QFileInfo editorFile(fileIFace->fileName());
-    QFileInfo changeFile(m_changeTmpFile->fileName());
+    QFileInfo changeFile(m_commitMessageFileName);
     if (editorFile.absoluteFilePath() == changeFile.absoluteFilePath()) {
         // Prompt the user. Force a prompt unless submit was actually invoked (that
         // is, the editor was closed or shutdown).
+        bool wantsPrompt = m_settings.promptToSubmit();
          const VCSBase::VCSBaseSubmitEditor::PromptSubmitResult answer =
             perforceEditor->promptSubmit(tr("Closing p4 Editor"),
                                          tr("Do you want to submit this change list?"),
                                          tr("The commit message check failed. Do you want to submit this change list"),
-                                         !m_submitActionTriggered);
+                                         &wantsPrompt, !m_submitActionTriggered);
          m_submitActionTriggered = false;
 
         if (answer == VCSBase::VCSBaseSubmitEditor::SubmitCanceled)
             return false;
 
+        // Set without triggering the checking mechanism
+        if (wantsPrompt != m_settings.promptToSubmit()) {
+            m_settings.setPromptToSubmit(wantsPrompt);
+            m_settings.toSettings(Core::ICore::instance()->settings());
+        }
         core->fileManager()->blockFileChange(fileIFace);
         fileIFace->save();
         core->fileManager()->unblockFileChange(fileIFace);
         if (answer == VCSBase::VCSBaseSubmitEditor::SubmitConfirmed) {
-            m_changeTmpFile->seek(0);
-            QByteArray change = m_changeTmpFile->readAll();
+            QFile commitMessageFile(m_commitMessageFileName);
+            if (!commitMessageFile.open(QIODevice::ReadOnly|QIODevice::Text)) {
+                VCSBase::VCSBaseOutputWindow::instance()->appendError(tr("Cannot open temporary file."));
+                return false;
+            }
+            QByteArray change = commitMessageFile.readAll();
+            commitMessageFile.close();
             QString errorMessage;
             if (!checkP4Configuration(&errorMessage)) {
-                showOutput(errorMessage, true);
+                VCSBase::VCSBaseOutputWindow::instance()->appendError(errorMessage);
                 return false;
             }
-
             QProcess proc;
             proc.setEnvironment(environment());
 
             QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
-            proc.start(m_settings.p4Command(),
-                m_settings.basicP4Args() << QLatin1String("submit") << QLatin1String("-i"));
+            QStringList submitArgs = m_settings.basicP4Args();
+            submitArgs << QLatin1String("submit") << QLatin1String("-i");
+            VCSBase::VCSBaseOutputWindow::instance()->appendCommand(formatCommand(m_settings.p4Command(), submitArgs));
+            proc.start(m_settings.p4Command(),submitArgs);
             if (!proc.waitForStarted(p4Timeout)) {
-                showOutput(tr("Cannot execute p4 submit."), true);
+                VCSBase::VCSBaseOutputWindow::instance()->appendError(tr("Cannot execute p4 submit."));
                 QApplication::restoreOverrideCursor();
                 return false;
             }
@@ -980,18 +1058,24 @@ bool PerforcePlugin::editorAboutToClose(Core::IEditor *editor)
             proc.closeWriteChannel();
 
             if (!proc.waitForFinished()) {
-                showOutput(tr("Cannot execute p4 submit."), true);
+                VCSBase::VCSBaseOutputWindow::instance()->appendError(tr("Cannot execute p4 submit."));
                 QApplication::restoreOverrideCursor();
                 return false;
             }
-            const QString output = QString::fromUtf8(proc.readAll());
-            showOutput(output);
+            const int exitCode = proc.exitCode();
+            if (exitCode) {
+                VCSBase::VCSBaseOutputWindow::instance()->appendError(tr("p4 submit failed (exit code %1).").arg(exitCode));
+                QApplication::restoreOverrideCursor();
+                return false;
+            }
+            const QString output = QString::fromLocal8Bit(proc.readAll());
+            VCSBase::VCSBaseOutputWindow::instance()->append(output);
             if (output.contains(QLatin1String("Out of date files must be resolved or reverted)"))) {
                 QMessageBox::warning(editor->widget(), tr("Pending change"), tr("Could not submit the change, because your workspace was out of date. Created a pending submit instead."));
             }
             QApplication::restoreOverrideCursor();
         }
-        cleanChangeTmpFile();
+        cleanCommitMessageFile();
     }
     return true;
 }
@@ -1079,49 +1163,8 @@ QString PerforcePlugin::pendingChangesData()
     return data;
 }
 
-void PerforcePlugin::showOutput(const QString &output, bool popup) const
-{
-    m_perforceOutputWindow->append(output, popup);
-}
-
 PerforcePlugin::~PerforcePlugin()
 {
-    if (m_settingsPage) {
-        removeObject(m_settingsPage);
-        delete m_settingsPage;
-        m_settingsPage = 0;
-    }
-
-    if (m_perforceOutputWindow) {
-        removeObject(m_perforceOutputWindow);
-        delete  m_perforceOutputWindow;
-        m_perforceOutputWindow = 0;
-    }
-
-    if (m_submitEditorFactory) {
-        removeObject(m_submitEditorFactory);
-        delete m_submitEditorFactory;
-        m_submitEditorFactory = 0;
-    }
-
-    if (m_versionControl) {
-        removeObject(m_versionControl);
-        delete m_versionControl;
-        m_versionControl = 0;
-    }
-
-    if (!m_editorFactories.empty()) {
-        foreach (Core::IEditorFactory *pf, m_editorFactories)
-            removeObject(pf);
-        qDeleteAll(m_editorFactories);
-        m_editorFactories.clear();
-    }
-
-    if (m_coreListener) {
-        removeObject(m_coreListener);
-        delete m_coreListener;
-        m_coreListener = 0;
-    }
 }
 
 const PerforceSettings& PerforcePlugin::settings() const
