@@ -23,7 +23,7 @@
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** If you are unsure which license is appropriate for your use, please
-** contact the sales department at http://www.qtsoftware.com/contact.
+** contact the sales department at http://qt.nokia.com/contact.
 **
 **************************************************************************/
 
@@ -40,6 +40,7 @@
 #include <coreplugin/uniqueidmanager.h>
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <utils/submiteditorwidget.h>
+#include <utils/checkablemessagebox.h>
 #include <utils/submitfieldwidget.h>
 #include <find/basetextfind.h>
 #include <texteditor/fontsettings.h>
@@ -47,6 +48,7 @@
 
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/session.h>
+#include <projectexplorer/project.h>
 
 #include <QtCore/QDebug>
 #include <QtCore/QDir>
@@ -64,6 +66,7 @@
 #include <QtGui/QMainWindow>
 #include <QtGui/QCompleter>
 #include <QtGui/QLineEdit>
+#include <QtGui/QTextEdit>
 
 enum { debug = 0 };
 enum { wantToolBar = 0 };
@@ -78,10 +81,10 @@ static inline QString submitMessageCheckScript()
 struct VCSBaseSubmitEditorPrivate
 {
     VCSBaseSubmitEditorPrivate(const VCSBaseSubmitEditorParameters *parameters,
-                               Core::Utils::SubmitEditorWidget *editorWidget,
+                               Utils::SubmitEditorWidget *editorWidget,
                                QObject *q);
 
-    Core::Utils::SubmitEditorWidget *m_widget;
+    Utils::SubmitEditorWidget *m_widget;
     QToolBar *m_toolWidget;
     const VCSBaseSubmitEditorParameters *m_parameters;
     QString m_displayName;
@@ -95,7 +98,7 @@ struct VCSBaseSubmitEditorPrivate
 };
 
 VCSBaseSubmitEditorPrivate::VCSBaseSubmitEditorPrivate(const VCSBaseSubmitEditorParameters *parameters,
-                                                       Core::Utils::SubmitEditorWidget *editorWidget,
+                                                       Utils::SubmitEditorWidget *editorWidget,
                                                        QObject *q) :
     m_widget(editorWidget),
     m_toolWidget(0),
@@ -107,7 +110,7 @@ VCSBaseSubmitEditorPrivate::VCSBaseSubmitEditorPrivate(const VCSBaseSubmitEditor
 }
 
 VCSBaseSubmitEditor::VCSBaseSubmitEditor(const VCSBaseSubmitEditorParameters *parameters,
-                                         Core::Utils::SubmitEditorWidget *editorWidget) :
+                                         Utils::SubmitEditorWidget *editorWidget) :
     m_d(new VCSBaseSubmitEditorPrivate(parameters, editorWidget, this))
 {
     // Message font according to settings
@@ -199,7 +202,7 @@ void VCSBaseSubmitEditor::createUserFields(const QString &fieldConfigFile)
     const QStandardItemModel *nickNameModel = Internal::VCSBasePlugin::instance()->nickNameModel();
     QCompleter *completer = new QCompleter(Internal::NickNameDialog::nickNameList(nickNameModel), this);
 
-    Core::Utils::SubmitFieldWidget *fieldWidget = new Core::Utils::SubmitFieldWidget;
+    Utils::SubmitFieldWidget *fieldWidget = new Utils::SubmitFieldWidget;
     connect(fieldWidget, SIGNAL(browseButtonClicked(int,QString)),
             this, SLOT(slotSetFieldNickName(int)));    
     fieldWidget->setCompleter(completer);
@@ -342,7 +345,7 @@ static QToolBar *createToolBar(const QWidget *someWidget, QAction *submitAction,
     return toolBar;
 }
 
-QToolBar *VCSBaseSubmitEditor::toolBar()
+QWidget *VCSBaseSubmitEditor::toolBar()
 {
     if (!wantToolBar)
         return 0;
@@ -433,12 +436,13 @@ VCSBaseSubmitEditor::PromptSubmitResult
         VCSBaseSubmitEditor::promptSubmit(const QString &title,
                                           const QString &question,
                                           const QString &checkFailureQuestion,
+                                          bool *promptSetting,
                                           bool forcePrompt) const
 {
     QString errorMessage;
     QMessageBox::StandardButton answer = QMessageBox::Yes;
 
-    const bool prompt = forcePrompt || Internal::VCSBasePlugin::instance()->settings().promptForSubmit;
+    const bool prompt = forcePrompt || *promptSetting;
 
     QWidget *parent = Core::ICore::instance()->mainWindow();
     // Pop up a message depending on whether the check succeeded and the
@@ -446,9 +450,19 @@ VCSBaseSubmitEditor::PromptSubmitResult
     if (checkSubmitMessage(&errorMessage)) {
         // Check ok, do prompt?
         if (prompt) {
-            answer = QMessageBox::question(parent, title, question,
-                                  QMessageBox::Yes|QMessageBox::No|QMessageBox::Cancel,
-                                  QMessageBox::Yes);
+            // Provide check box to turn off prompt ONLY if it was not forced
+            if (*promptSetting && !forcePrompt) {
+                const QDialogButtonBox::StandardButton danswer =
+                        Utils::CheckableMessageBox::question(parent, title, question,
+                                                                   tr("Prompt to submit"), promptSetting,
+                                                                   QDialogButtonBox::Yes|QDialogButtonBox::No|QDialogButtonBox::Cancel,
+                                                                   QDialogButtonBox::Yes);
+                answer = Utils::CheckableMessageBox::dialogButtonBoxToMessageBoxButton(danswer);
+            } else {
+                answer = QMessageBox::question(parent, title, question,
+                                               QMessageBox::Yes|QMessageBox::No|QMessageBox::Cancel,
+                                               QMessageBox::Yes);
+            }
         }
     } else {
         // Check failed.
@@ -488,7 +502,7 @@ void VCSBaseSubmitEditor::slotInsertNickName()
 
 void VCSBaseSubmitEditor::slotSetFieldNickName(int i)
 {
-    if (Core::Utils::SubmitFieldWidget *sfw  =m_d->m_widget->submitFieldWidgets().front()) {
+    if (Utils::SubmitFieldWidget *sfw  =m_d->m_widget->submitFieldWidgets().front()) {
         const QString nick = promptForNickName();
         if (!nick.isEmpty())
             sfw->setFieldValue(i, nick);
@@ -564,6 +578,7 @@ QIcon VCSBaseSubmitEditor::submitIcon()
     return QIcon(QLatin1String(":/vcsbase/images/submit.png"));
 }
 
+// Compile a list if files in the current projects. TODO: Recurse down qrc files?
 QStringList VCSBaseSubmitEditor::currentProjectFiles(bool nativeSeparators, QString *name)
 {
     using namespace ProjectExplorer;
@@ -594,6 +609,26 @@ QStringList VCSBaseSubmitEditor::currentProjectFiles(bool nativeSeparators, QStr
     return files;
 }
 
+// Reduce a list of untracked files reported by a VCS down to the files
+// that are actually part of the current project(s).
+void VCSBaseSubmitEditor::filterUntrackedFilesOfProject(const QString &repositoryDirectory, QStringList *untrackedFiles)
+{
+    if (untrackedFiles->empty())
+        return;
+    const QStringList nativeProjectFiles = VCSBase::VCSBaseSubmitEditor::currentProjectFiles(true);
+    if (nativeProjectFiles.empty())
+        return;
+    const QDir repoDir(repositoryDirectory);
+    for (QStringList::iterator it = untrackedFiles->begin(); it != untrackedFiles->end(); ) {
+        const QString path = QDir::toNativeSeparators(repoDir.absoluteFilePath(*it));
+        if (nativeProjectFiles.contains(path)) {
+            ++it;
+        } else {
+            it = untrackedFiles->erase(it);
+        }
+    }
+}
+
 // Helper to raise an already open submit editor to prevent opening twice.
 bool VCSBaseSubmitEditor::raiseSubmitEditor()
 {
